﻿Namespace AdvSoft.UtilitiesLayer

    ''' <summary>
    ''' Utilidades relacionadas directamente con ASP
    ''' </summary>
    ''' <remarks></remarks>
    Public NotInheritable Class AspUtilites

        ' Visibilidad privata para ocultar el constructor evitando poder crear un objeto de la clase 
        Private Sub New()
        End Sub

        ''' <summary>
        ''' Retorna la dirección completa de la aplicación web. Ejemplo: http://192.168.0.1:4002/appdir
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Shared Function FullyQualifiedApplicationPath() As String

            ' Return variable declaration
            Dim appPath As String = String.Empty

            ' Getting the current context of HTTP request
            Dim context As HttpContext = HttpContext.Current

            ' Checking the current context content
            If Not context Is Nothing Then
                ' Formatting the fully qualified website url/name
                appPath = String.Format("{0}://{1}{2}{3}", _
                                        context.Request.Url.Scheme, _
                                        context.Request.Url.Host, _
                                        If(context.Request.Url.Port = 80, String.Empty, ":" & context.Request.Url.Port), _
                                        context.Request.ApplicationPath)
            End If

            If Not appPath.EndsWith("/") Then appPath += "/"

            Return appPath
        End Function

        ''' <summary>
        ''' Retorna la ip del cliente conectado a la aplicación
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Shared Function GetClientIP() As String

            ' Return variable declaration
            Dim clientIP As String = String.Empty

            'clientIP = HttpContext.Current.Request.UserHostAddress

            ' El visitante puede acceder por proxy, entonces tomo la IP que lo está utilizando
            clientIP = HttpContext.Current.Request.ServerVariables("HTTP_X_FORWARDED_FOR")

            ' Si no venía de un proxy, tomo la ip del visitante
            If clientIP Is Nothing OrElse clientIP.Length = 0 Then
                clientIP = HttpContext.Current.Request.ServerVariables("REMOTE_ADDR").ToString
            End If

            Return clientIP
        End Function

        ''' <summary>
        ''' Retorna un javascript con un mensaje de confirmación y redirección a una página tras la aceptación del mensaje
        ''' </summary>
        ''' <param name="mensaje">Mensaje ha mostrar en el cuadro de confirmación</param>
        ''' <param name="redirectPage">Página a la que ir tras aceptar el mensaje de confirmación. Por defecto a default.aspx</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Shared Function ClientSideScript_ConfirmMsgAndRedirect(ByVal mensaje As String, Optional ByVal redirectPage As String = "default.aspx") As String

            Dim scripTest As New StringBuilder
            scripTest.AppendFormat("confirm('{0}');", mensaje)
            scripTest.AppendLine()
            scripTest.AppendFormat("window.location.href = '{0}{1}'", FullyQualifiedApplicationPath, redirectPage)

            Return scripTest.ToString
        End Function

    End Class

End Namespace