﻿Imports AppGestio.ModelLayer

Public Class General
    'Public Shared client As ReformesServiceClient
    Public Shared ctx As New DataLayer.ServiceWCFContext






#Region "Usuaris"


    Public Shared Function Login(ByRef Usuari As Usuari) As Boolean

    End Function

    ''' <summary>
    ''' Obté la llista d'usuaris donats d'alta a l'aplicació
    ''' </summary>
    ''' <param name="orden"></param>
    ''' <param name="filtronombre"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetUsuarisList(ByVal orden As String, ByVal filtronombre As String) As List(Of Usuari)
        Try
            Dim q = (From u As Usuari In ctx.USUARIS Select u)

            If orden Is Nothing Then orden = ""

            If Not String.IsNullOrEmpty(filtronombre) Then
                q = q.Where(Function(x) x.Nom.ToLower.Contains(filtronombre.ToLower))
            End If

            Select Case orden.ToLower
                Case "id_desc"
                    q = q.OrderByDescending(Function(c) c.UsuariId)
                Case "nom"
                    q = q.OrderBy(Function(c) c.Nom)
                Case "nom_desc"
                    q = q.OrderByDescending(Function(c) c.Nom)
                Case "dataalta"
                    q = q.OrderBy(Function(c) c.DataAlta)
                Case "dataalta_desc"
                    q = q.OrderByDescending(Function(c) c.DataAlta)
                Case Else
                    q = q.OrderBy(Function(c) c.UsuariId)
            End Select

            Return q.ToList

        Catch ex As Exception
            Return New List(Of ModelLayer.Usuari)
        End Try
    End Function

    ''' <summary>
    ''' Obté un usuari a partir del seu Id
    ''' </summary>
    ''' <param name="usuariID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetUsuariById(usuariID As Long) As Usuari
        Return (From x As Usuari In General.ctx.USUARIS Where x.UsuariId = usuariID).FirstOrDefault
    End Function

    ''' <summary>
    ''' Obté un usuari a partir del seu nom
    ''' </summary>
    ''' <param name="UserName"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetUsuariByName(UserName As String) As Usuari
        Return (From x As Usuari In General.ctx.USUARIS Where x.NomUsuari.ToLower = UserName.ToLower).FirstOrDefault
    End Function

    ''' <summary>
    ''' Crea o modifica un usuari a la base de dades
    ''' </summary>
    ''' <param name="usuari"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(usuari As Usuari) As Boolean
        Dim u As Usuari

        If usuari.UsuariId < 0 Then
            u = usuari
            General.ctx.USUARIS.Add(u)
        Else
            u = (From x As Usuari In General.ctx.USUARIS Where x.UsuariId = usuari.UsuariId).FirstOrDefault
            If u Is Nothing Then
                u = usuari
                General.ctx.USUARIS.Add(u)
            Else
                u.FromObject(usuari)
            End If
        End If

        ctx.SaveChanges()

        Return True
    End Function

    ''' <summary>
    ''' Elimina un usuari de la base de dades
    ''' </summary>
    ''' <param name="usuariId">Id de l'usuari a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarUsuari(usuariId As Integer) As Boolean
        Dim u As Usuari
        u = (From x As Usuari In General.ctx.USUARIS Where x.UsuariId = usuariId).FirstOrDefault
        If u IsNot Nothing Then
            General.ctx.USUARIS.Remove(u)
        End If
        Return True
    End Function

    ''' <summary>
    ''' Retorna true si un usuari amb el nom especificat es del tipus administrador
    ''' </summary>
    ''' <param name="UserName"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function IsAdmin(UserName As String) As Boolean

        Try
            Dim usr As Usuari = GetUsuariByName(UserName)

            'Els usuaris "admin" i "administrador" sempre son administradors
            If usr.NomUsuari.ToLower = "administrador" OrElse usr.Nom.ToLower = "admin" Then
                Return True
            End If

            Dim t As EnumTipusUsuari? = (usr.Tipus)
            Return If(t.HasValue, (t.Value = EnumTipusUsuari.Administrador), False)
        Catch ex As Exception
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Retorna 'true' si existeix algun usuari administrador
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    ''' 
    Public Shared Function ExistsAdmin() As Boolean
        If ctx.USUARIS.Count = 0 Then
            Return False
        Else
            Return (From u As Usuari In ctx.USUARIS Where u.Tipus = EnumTipusUsuari.Administrador).Count > 0
        End If
    End Function

#End Region

#Region "Gestió Clients"

    ''' <summary>
    ''' Obté la llista de clients de la base de dades
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetClientsList(ByVal orden As String, ByVal filtronombre As String) As List(Of Client)

        'Return ModelLayer.Client.GetSampleList
        Try
            Dim q = (From c As Client In ctx.CLIENTS Select c)

            If orden Is Nothing Then orden = ""

            If Not String.IsNullOrEmpty(filtronombre) Then
                q = q.Where(Function(x) x.Nom.ToLower.Contains(filtronombre.ToLower) Or
                              x.NIF.ToLower.Contains(filtronombre.ToLower))
            End If

            Select Case orden.ToLower
                Case "id_desc"
                    q = q.OrderByDescending(Function(c) c.ClientId)
                Case "nom"
                    q = q.OrderBy(Function(c) c.Nom)
                Case "nom_desc"
                    q = q.OrderByDescending(Function(c) c.Nom)
                Case "nif"
                    q = q.OrderBy(Function(c) c.NIF)
                Case "nif_desc"
                    q = q.OrderByDescending(Function(c) c.NIF)
                Case Else
                    q = q.OrderBy(Function(c) c.ClientId)
            End Select

            Return q.ToList

        Catch ex As Exception
            Return New List(Of ModelLayer.Client)
        End Try

    End Function

    ''' <summary>
    ''' Obté un client a partir del seu Id
    ''' </summary>
    ''' <param name="clientID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetClientById(clientID As Long) As Client
        Return General.ctx.CLIENTS.Find(clientID)
    End Function

    ''' <summary>
    ''' Crea o modifica un client a la base de dades
    ''' </summary>
    ''' <param name="client"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(client As Client) As Boolean
        Dim c As Client

        If client.ClientId <= 0 Then
            c = client
            General.ctx.CLIENTS.Add(c)
        Else

            c = General.ctx.CLIENTS.Find(client.ClientId)
            If c Is Nothing Then
                c = client
                General.ctx.CLIENTS.Add(c)
            Else
                c.FromObject(client)
            End If

            ctx.Entry(c).State = EntityState.Modified
        End If


        General.ctx.SaveChanges()

        Return True
    End Function

    ''' <summary>
    ''' Elimina un client de la base de dades
    ''' </summary>
    ''' <param name="clientId">Id del client a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarClient(clientId As Long) As Boolean
        Dim c As Client
        c = (From x As Client In General.ctx.CLIENTS Where x.ClientId = clientId).FirstOrDefault
        If c IsNot Nothing Then
            General.ctx.CLIENTS.Remove(c)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

#End Region

#Region "Gestió Pressupostos"

    Friend Shared Function GetNewIdPressupost() As Long
        If ctx.PRESSUPOSTOS.Count = 0 Then
            Return 1
        Else
            Return (From x As Pressupost In ctx.PRESSUPOSTOS Select x.PressupostId).Max + 1
        End If
    End Function

    ''' <summary>
    ''' Retorna la llista de pressupostos que es troben en un estat concret
    ''' </summary>
    ''' <param name="estat">Filtre d'estat</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetpressupostosByEstat(estat As EnumEstatsPressupost) As List(Of Pressupost)

        'Return (From x As Pressupost In Pressupost.GetSampleList Where x.EstatPressupost = estat).ToList

        Return (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.EstatPressupost = estat).ToList

    End Function

    ''' <summary>
    ''' Retorna la llista de pressupostos entre les dates seleccionades
    ''' </summary>
    ''' <param name="DataInicial">Data inicial</param>
    ''' <param name="DataFinal">Data Final</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetpressupostosList(DataInicial As Date, DataFinal As Date) As List(Of Pressupost)


        'Return (From x As Pressupost In Pressupost.GetSampleList Where x.Data >= New Date(DataInicial.Year, DataInicial.Month, DataInicial.Day, 0, 0, 0) And
        '        x.Data <= New Date(DataFinal.Year, DataFinal.Month, DataFinal.Day, 23, 59, 59)).ToList

        Dim d1 As Date = New Date(DataInicial.Year, DataInicial.Month, DataInicial.Day, 0, 0, 0)
        Dim d2 As Date = New Date(DataFinal.Year, DataFinal.Month, DataFinal.Day, 23, 59, 59)

        Return (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.Data >= d1 And
                x.Data <= d2).ToList

    End Function

    ''' <summary>
    ''' Retorna la llista de pressupostos d'un client concret
    ''' </summary>
    ''' <param name="idClient">Identificador del client</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetpressupostosList(idClient As Integer) As List(Of Pressupost)

        'Return (From x As Pressupost In Pressupost.GetSampleList Where x.Client IsNot Nothing AndAlso x.Client.ClientId = idClient).ToList

        Return (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.Client IsNot Nothing AndAlso x.Client.ClientId = idClient).ToList
    End Function

    Public Shared Function GetPressupostById(pressupostID As Long) As Pressupost

        'Return (From x As Pressupost In Pressupost.GetSampleList Where x.PressupostId = pressupostID).FirstOrDefault

        Return (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.PressupostId = pressupostID).FirstOrDefault
    End Function

    Public Shared Function PressupostAddLinea(pressupost As Pressupost) As Boolean

        Try
            Dim xLin As Integer = 1
            If pressupost Is Nothing Then
                Return False
            End If

            pressupost = GetPressupostById(pressupost.PressupostId)

            If pressupost.LiniesPressupost Is Nothing Then
                pressupost.LiniesPressupost = New List(Of ModelLayer.PressupostLin)
            ElseIf pressupost.LiniesPressupost.Count > 0 Then
                xLin = (From x As ModelLayer.PressupostLin In pressupost.LiniesPressupost Select x.Linia).Max + 1
            End If

            If pressupost.PressupostId < 0 Then
                Actualitza(pressupost)
            End If

            pressupost.LiniesPressupost.Add(New ModelLayer.PressupostLin With {.Linia = CByte(xLin),
                                           .PressupostId = pressupost.PressupostId,
                                           .Pressupost = pressupost,
                                           .Quantitat = 1,
                                           .Preu = 0})

            ' actualitzar el pressupost a la base de dades
            Actualitza(pressupost)

            Return True
        Catch ex As Exception
            Return False
        End Try

    End Function

    ''' <summary>
    ''' Crea o modifica un pressupost a la base de dades
    ''' </summary>
    ''' <param name="pressupost">Pressupost a actualitzar (si idPressupost=-1, s'afegeix)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(pressupost As Pressupost) As Boolean
        Dim p As Pressupost

        Try
            If pressupost.PressupostId < 0 Then
                p = pressupost
                p.PressupostId = General.GetNewIdPressupost
                p.ClientId = (From c As Client In ctx.CLIENTS Select c.ClientId).FirstOrDefault
                General.ctx.PRESSUPOSTOS.Add(p)
            Else
                p = (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.PressupostId = pressupost.PressupostId).FirstOrDefault
                If p Is Nothing Then
                    p = pressupost
                    p.PressupostId = General.GetNewIdPressupost
                    General.ctx.PRESSUPOSTOS.Add(p)
                Else
                    p.FromObject(pressupost)
                End If
            End If

            General.ctx.SaveChanges()

            Return True
        Catch ex As Exception
            Return False
        End Try

    End Function

    Public Shared Function Actualitza(Lin As PressupostLin) As Boolean
        If Lin Is Nothing Then Return False
        Dim OldLin As PressupostLin
        Dim p As Pressupost = GetPressupostById(Lin.PressupostId)


        'Si Lin = -1, crear nova, si no, actualitzar.

        OldLin = (From L As PressupostLin In p.LiniesPressupost Where L.Linia = Lin.Linia).FirstOrDefault

        If Lin.Linia = -1 OrElse OldLin Is Nothing Then
            Try
                Lin.Linia = CByte((From L As PressupostLin In p.LiniesPressupost
                                        Select L.Linia).Max + 1)
            Catch ex As Exception
                Lin.Linia = 1
            End Try

            p.LiniesPressupost.Add(Lin)

            ctx.Entry(p).State = EntityState.Modified

        Else
            OldLin.ArticleId = Lin.ArticleId
            OldLin.Quantitat = Lin.Quantitat
            OldLin.Preu = Lin.Preu
            OldLin.PercIVA = Lin.PercIVA
            ctx.Entry(p).State = EntityState.Modified
        End If


        General.ctx.SaveChanges()
        Return True

    End Function

    Public Shared Function EliminarLiniaPresupost(PressupostId As Long, Linia As Byte) As Boolean

        Dim p As Pressupost = GetPressupostById(PressupostId)
        If p Is Nothing Then Return False
        Dim l As PressupostLin = (From xlin As PressupostLin In p.LiniesPressupost Where xlin.Linia = Linia).FirstOrDefault
        If l Is Nothing Then Return False
        Dim ok As Boolean = p.LiniesPressupost.Remove(l)
        ctx.Entry(p).State = EntityState.Modified
        ctx.SaveChanges()
        Return ok
    End Function

    ''' <summary>
    ''' Elimina un pressupost de la base de dades
    ''' </summary>
    ''' <param name="pressupostId">Id del pressupost a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarPressupost(pressupostId As Integer) As Boolean
        Dim p As Pressupost
        p = (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.PressupostId = pressupostId).FirstOrDefault
        If p IsNot Nothing Then
            General.ctx.PRESSUPOSTOS.Remove(p)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Accepta un pressuposti genera la factura vinculada 
    ''' </summary>
    ''' <param name="pressupostId"></param>
    ''' <returns>Retorna la factura generada a conseqüència d' acceptar el pressupost </returns>
    ''' <remarks></remarks>
    Public Shared Function AcceptarPressupost(pressupostId As Integer) As FacturaCli
        Dim p As Pressupost
        p = (From x As Pressupost In General.ctx.PRESSUPOSTOS Where x.PressupostId = pressupostId).FirstOrDefault

        Dim F As FacturaCli = Nothing


        Select Case p.EstatPressupost
            Case EnumEstatsPressupost.Acceptat
                Return Nothing
            Case EnumEstatsPressupost.Generat
                'Com está en estat generat, es pot acceptar. 
                'Crear una factura amb les mateixes dades que el pressupost 
                F = New FacturaCli

                F.FacturaCliId = General.GetNewIdFactura
                F.Data = Now
                F.EstatFactura = EnumEstatsFactura.Generada
                F.Pressupost = p

                F.Client = p.Client
                F.Descripcio = p.Descripcio
                F.FormaPagament = EnumFormesPagament.Comptat
                F.NumVenciments = p.NumVenciments
                F.PercDescompte = p.PercDescompte
                F.Usuari = p.Usuari
                F.LiniesFactura = New List(Of FacturaCliLin)
                For Each L As PressupostLin In p.LiniesPressupost
                    F.LiniesFactura.Add(New FacturaCliLin With {.Article = L.Article,
                                                                .ArticleId = L.ArticleId,
                                                                .Descripcio = If(L.Descripcio Is Nothing, "", L.Descripcio),
                                                                .FacturaCli = F,
                                                                .FacturaCliId = F.FacturaCliId,
                                                                .PercDescompte = L.PercDescompte,
                                                                .PercIVA = L.PercIVA,
                                                                .Preu = L.Preu,
                                                                .Quantitat = L.Quantitat,
                                                                .Unitats = L.Unitats,
                                                               .Linia = L.Linia})
                Next

                p.EstatPressupost = EnumEstatsPressupost.Acceptat
                ctx.Entry(p).State = EntityState.Modified

                ctx.FACTURES_CLI.Add(F)

                ctx.SaveChanges()

        End Select


        Return F
    End Function

#End Region

#Region "Gestió Factures"

    Friend Shared Function GetNewIdFactura() As Long
        If ctx.FACTURES_CLI.Count = 0 Then
            Return 1
        Else
            Return (From x As FacturaCli In ctx.FACTURES_CLI Select x.FacturaCliId).Max + 1
        End If
    End Function

    ''' <summary>
    ''' Retorna la llista de factures que es troben en un estat concret
    ''' </summary>
    ''' <param name="estat">Filtre d'estat</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesClientByEstat(estat As EnumEstatsFactura) As List(Of FacturaCli)
        Return (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.EstatFactura = estat).ToList
    End Function

    ''' <summary>
    ''' Retorna la llista de factures entre les dates seleccionades
    ''' </summary>
    ''' <param name="DataInicial">Data inicial</param>
    ''' <param name="DataFinal">Data Final</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesList(DataInicial As Date, DataFinal As Date) As List(Of FacturaCli)

        Dim d1 As Date = New Date(DataInicial.Year, DataInicial.Month, DataInicial.Day, 0, 0, 0)
        Dim d2 As Date = New Date(DataFinal.Year, DataFinal.Month, DataFinal.Day, 23, 59, 59)

        Return (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.Data >= d1 And
                x.Data <= d2).ToList
    End Function

    ''' <summary>
    ''' Retorna la llista de factures d'un client concret
    ''' </summary>
    ''' <param name="idClient">Identificador del client</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesList(idClient As Integer) As List(Of FacturaCli)
        Return (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.Client IsNot Nothing AndAlso x.Client.ClientId = idClient).ToList
    End Function

    ''' <summary>
    ''' Obté una factura a partir del seu Id
    ''' </summary>
    ''' <param name="facturaCliID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetFacturaCliById(facturaCliID As Long) As FacturaCli
        Return (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = facturaCliID).FirstOrDefault
    End Function

    ''' <summary>
    ''' Crea o modifica una factura a la base de dades
    ''' </summary>
    ''' <param name="factura">Factura a actualitzar (si idFactura=-1, s'afegeix)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(factura As FacturaCli) As Boolean
        Dim f As FacturaCli

        If factura.FacturaCliId < 0 Then
            f = factura
            f.FacturaCliId = General.GetNewIdFactura
            General.ctx.FACTURES_CLI.Add(f)
        Else
            f = (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = factura.FacturaCliId).FirstOrDefault
            If f Is Nothing Then
                f = factura
                f.FacturaCliId = General.GetNewIdFactura
                General.ctx.FACTURES_CLI.Add(f)
            Else
                f.FromObject(factura)
            End If
        End If

        General.ctx.SaveChanges()

        Return True
    End Function

    ''' <summary>
    ''' Elimina una factura de la base de dades
    ''' </summary>
    ''' <param name="facturaId">Id de la factura a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarFactura(facturaId As Integer) As Boolean
        Dim f As FacturaCli
        f = (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            General.ctx.FACTURES_CLI.Remove(f)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Si la factura està en estat 'Generada' la passa a l'estat 'Acceptada'
    ''' </summary>
    ''' <param name="facturaId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function AcceptarFacturaClient(facturaId As Integer) As FacturaCli
        Dim f As FacturaCli
        f = (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            If f.EstatFactura = EnumEstatsFactura.Generada Then
                f.EstatFactura = EnumEstatsFactura.Acceptada
                General.ctx.SaveChanges()
            End If
        End If


        Return f
    End Function

    ''' <summary>
    ''' Passa la factura a estat 'Facturada' i genera els cobraments en base a la informació de la factura 
    ''' </summary>
    ''' <param name="facturaId"></param>
    ''' <returns>Retorna la factura actualitzada </returns>
    ''' <remarks></remarks>
    Public Shared Function FacturarFacturaClient(facturaId As Integer) As FacturaCli
        Dim f As FacturaCli
        f = (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            If f.EstatFactura = EnumEstatsFactura.Acceptada Then
                'Generar los cobros

                f.Cobraments = New List(Of Cobrament)
                Dim suma As Double = 0
                Dim imp As Double

                For i As Integer = 1 To f.NumVenciments
                    If i < f.NumVenciments Then
                        imp = ((f.Import * 100) \ f.NumVenciments) / 100
                        suma += imp
                    Else
                        ' Si es l'ultim venciment, s'haurà de pagar la resta que quedi pendent
                        imp = f.Import - suma

                    End If

                    Dim dv As Date = f.Data.AddMonths(i - 1)
                    dv = DateSerial(dv.Year, dv.Month, dv.Day)

                    Dim c As New Cobrament With {
                                         .Client = f.Client,
                                         .ClientId = f.Client.ClientId,
                                         .DataFactura = f.Data,
                                         .DataVenciment = dv,
                                         .UltimaReclamacio = Date.MinValue,
                                         .Estat = EnumEstatsTransaccio.NoCobrat,
                                         .FacturaCli = f,
                                         .FacturaCliId = f.FacturaCliId,
                                         .FormaPagament = f.FormaPagament,
                                         .Numero = i,
                                         .Import = imp}



                    f.Cobraments.Add(c)
                Next

                f.EstatFactura = EnumEstatsFactura.Facturada
                General.ctx.SaveChanges()
            End If
        End If
        Return f
    End Function

    ''' <summary>
    ''' Si la l'estat de la factura és 'Facturada', passa a l'estat 'Acceptada' i s'eliminen els cobraments.
    ''' Si l'estat es 'Acceptada', passa a l'estat 'Generada'.
    ''' </summary>
    ''' <param name="facturaId"></param>
    ''' <returns>Retorna la factura actualitzada </returns>
    ''' <remarks>Si la factura té cobraments en estat 'Cobrat', no es canviarà l'estat. Primer s'han de canviar els estats dels cobraments.</remarks>
    Public Shared Function AnularFacturaClient(facturaId As Integer) As FacturaCli
        Dim f As FacturaCli
        f = (From x As FacturaCli In General.ctx.FACTURES_CLI Where x.FacturaCliId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            If f.EstatFactura = EnumEstatsFactura.Acceptada Then
                'Si està en estat acceptada, passa a l'estat Generada.
                f.EstatFactura = EnumEstatsFactura.Generada

                General.ctx.SaveChanges()
            ElseIf f.EstatFactura = EnumEstatsFactura.Facturada Then
                'Si està en estat facturada, no té cobraments realitzats encara. Llavors, es poden eliminar els cobraments i
                'passar a l'estat Acceptada

                If f.Cobraments IsNot Nothing Then
                    Dim LC As New List(Of Cobrament)

                    For Each c As Cobrament In f.Cobraments
                        LC.Add(c)
                    Next

                    For Each c As Cobrament In LC
                        General.ctx.COBRAMENTS.Remove(c)
                    Next

                End If

                f.Cobraments = New List(Of Cobrament)

                f.EstatFactura = EnumEstatsFactura.Acceptada

                General.ctx.SaveChanges()
            End If
        End If

        Return f
    End Function

#End Region

#Region "Gestió Cobraments"

    ''' <summary>
    ''' Retorna la llista de cobraments entre les dates seleccionades.
    ''' </summary>
    ''' <param name="DataInicial">Data inicial</param>
    ''' <param name="DataFinal">Data Final</param>
    ''' <param name="estat">(Opcional) Estat del cobrament. Si no s'especifica, es retornen tots</param>
    ''' <param name="clientId">(Opcional) Filtre per client. Si no s'especifica, es retornen tots</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetcobramentsList(DataInicial As Date, DataFinal As Date, Optional estat As EnumEstatsTransaccio = 255, Optional clientId As Integer = -1) As List(Of Cobrament)
        Dim q = (From x As Cobrament In General.ctx.COBRAMENTS Where x.DataVenciment >= New Date(DataInicial.Year, DataInicial.Month, DataInicial.Day, 0, 0, 0) And
                x.DataVenciment <= New Date(DataFinal.Year, DataFinal.Month, DataFinal.Day, 23, 59, 59))

        If estat <> 255 Then
            q = (From x As Cobrament In q
                Where x.Estat = estat)
        End If

        If clientId <> -1 Then
            q = (From x As Cobrament In q
                Where x.ClientId = clientId)
        End If

        Return q.ToList
    End Function

    ''' <summary>
    ''' Crea o modifica un cobrament a la base de dades
    ''' </summary>
    ''' <param name="cobrament">Cobrament a actualitzar (si id=-1, s'afegeix)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(cobrament As Cobrament) As Boolean
        Dim c As Cobrament

        If cobrament.Id < 0 Then
            c = cobrament
            General.ctx.COBRAMENTS.Add(c)
        Else
            c = (From x As Cobrament In General.ctx.COBRAMENTS Where x.Id = cobrament.Id).FirstOrDefault
            If c Is Nothing Then
                c = cobrament
                General.ctx.COBRAMENTS.Add(c)
            Else
                c.FromObject(cobrament)
            End If
        End If

        General.ctx.SaveChanges()

        Return True
    End Function

    ''' <summary>
    ''' Obté un cobrament a partir del seu Id
    ''' </summary>
    ''' <param name="cobramentID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetCobramentsById(cobramentID As Long) As Cobrament
        Return (From x As Cobrament In General.ctx.COBRAMENTS Where x.Id = cobramentID).FirstOrDefault
    End Function

    ''' <summary>
    ''' Elimina un cobrament de la base de dades
    ''' </summary>
    ''' <param name="cobramentId">Id del cobrament a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarCobrament(cobramentId As Integer) As Boolean
        Dim c As Cobrament
        c = (From x As Cobrament In General.ctx.COBRAMENTS Where x.Id = cobramentId).FirstOrDefault
        If c IsNot Nothing Then
            General.ctx.COBRAMENTS.Remove(c)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Passa el cobrament a estat 'Processat' i canvia l'estat de la factura en funció del nombre de cobraments pendents. 
    ''' </summary>
    ''' <param name="cobramentId"></param>
    ''' <returns>Retorna la factura actualitzada </returns>
    ''' <remarks></remarks>
    Public Shared Function ProcessarCobrament(cobramentId As Integer) As Cobrament
        Dim c As Cobrament
        c = (From x As Cobrament In General.ctx.COBRAMENTS Where x.Id = cobramentId).FirstOrDefault

        If c IsNot Nothing Then
            c.Estat = EnumEstatsTransaccio.Cobrat

            If (From x As Cobrament In c.FacturaCli.Cobraments
                Where x.Estat = EnumEstatsTransaccio.NoCobrat).Count = 0 Then

                c.FacturaCli.EstatFactura = EnumEstatsFactura.Cobrada
            Else
                c.FacturaCli.EstatFactura = EnumEstatsFactura.CobradaParcialment
            End If

            General.ctx.SaveChanges()
        End If

        Return c
    End Function

    ''' <summary>
    ''' Si la l'estat del cobrament és 'Processat', passa a l'estat 'No Processat' en cas contrari, no té efecte.
    ''' </summary>
    ''' <param name="cobramentId"></param>
    ''' <returns>Retorna el cobrament actualitzat </returns>
    ''' <remarks>Si la factura té cobraments en estat 'Cobrat', no es canviarà l'estat. Primer s'han de canviar els estats dels cobraments.</remarks>
    Public Shared Function CancelarCobrament(cobramentId As Integer) As Cobrament
        Dim c As Cobrament
        c = (From x As Cobrament In General.ctx.COBRAMENTS Where x.Id = cobramentId).FirstOrDefault

        If c IsNot Nothing Then
            c.Estat = EnumEstatsTransaccio.NoCobrat

            If (From x As Cobrament In c.FacturaCli.Cobraments
                Where x.Estat = EnumEstatsTransaccio.Cobrat).Count = 0 Then

                c.FacturaCli.EstatFactura = EnumEstatsFactura.CobradaParcialment
            Else
                c.FacturaCli.EstatFactura = EnumEstatsFactura.Facturada
            End If

            General.ctx.SaveChanges()
        End If

        Return c
    End Function

#End Region

#Region "Gestió Proveïdor"

    ''' <summary>
    ''' Obté la llista de proveidors de la base de dades
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetProveidorList(ByVal orden As String, ByVal filtronombre As String) As List(Of Proveidor)
        Try
            Dim q = (From c As Proveidor In ctx.PROVEIDORS Select c)

            If orden Is Nothing Then orden = ""

            If Not String.IsNullOrEmpty(filtronombre) Then
                q = q.Where(Function(x) x.Nom.ToLower.Contains(filtronombre.ToLower) Or
                              x.NIF.ToLower.Contains(filtronombre.ToLower))
            End If

            Select Case orden.ToLower
                Case "id_desc"
                    q = q.OrderByDescending(Function(c) c.ProveidorId)
                Case "nom"
                    q = q.OrderBy(Function(c) c.Nom)
                Case "nom_desc"
                    q = q.OrderByDescending(Function(c) c.Nom)
                Case "nif"
                    q = q.OrderBy(Function(c) c.NIF)
                Case "nif_desc"
                    q = q.OrderByDescending(Function(c) c.NIF)
                Case Else
                    q = q.OrderBy(Function(c) c.ProveidorId)
            End Select

            Return q.ToList

        Catch ex As Exception
            Return New List(Of ModelLayer.Proveidor)
        End Try
    End Function

    ''' <summary>
    ''' Obté un proveïdor a partir del seu Id
    ''' </summary>
    ''' <param name="proveidorID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetProveidorById(proveidorID As Long) As Proveidor
        Return General.ctx.PROVEIDORS.Find(proveidorID)
    End Function

    ''' <summary>
    ''' Crea o modifica un proveïdor a la base de dades
    ''' </summary>
    ''' <param name="proveidor"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(proveidor As Proveidor) As Boolean
        Dim p As Proveidor

        If proveidor.ProveidorId <= 0 Then
            p = proveidor
            General.ctx.PROVEIDORS.Add(p)
        Else

            p = General.ctx.PROVEIDORS.Find(proveidor.ProveidorId)
            If p Is Nothing Then
                p = proveidor
                General.ctx.PROVEIDORS.Add(p)
            Else
                p.FromObject(proveidor)
            End If

            ctx.Entry(p).State = EntityState.Modified
        End If


        General.ctx.SaveChanges()

        Return True
    End Function

    ''' <summary>
    ''' Elimina un proveidor de la base de dades
    ''' </summary>
    ''' <param name="proveidorId">Id del proveidor a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarPRoveidor(proveidorId As Integer) As Boolean
        Dim p As Proveidor
        p = (From x As Proveidor In General.ctx.PROVEIDORS Where x.ProveidorId = proveidorId).FirstOrDefault
        If p IsNot Nothing Then
            General.ctx.PROVEIDORS.Remove(p)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

#End Region

#Region "Gestió Factures de Proveïdor"


    Friend Shared Function GetNewIdFacturaPro() As Long
        If ctx.FACTURES_PRO.Count = 0 Then
            Return 1
        Else
            Return (From x As FacturaPro In ctx.FACTURES_PRO Select x.FacturaProId).Max + 1
        End If
    End Function

    ''' <summary>
    ''' Retorna la llista de factures de proveidor entre les dates seleccionades
    ''' </summary>
    ''' <param name="DataInicial">Data inicial</param>
    ''' <param name="DataFinal">Data Final</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesProList(DataInicial As Date, DataFinal As Date) As List(Of FacturaPro)
        Dim d1 As Date = New Date(DataInicial.Year, DataInicial.Month, DataInicial.Day, 0, 0, 0)
        Dim d2 As Date = New Date(DataFinal.Year, DataFinal.Month, DataFinal.Day, 23, 59, 59)

        Return (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.Data >= d1 And
                x.Data <= d2).ToList
    End Function

    ''' <summary>
    ''' Retorna la llista de factures que es troben en un estat concret
    ''' </summary>
    ''' <param name="estat">Filtre d'estat</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesProByEstat(estat As EnumEstatsFactura) As List(Of FacturaPro)
        Return (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.EstatFactura = estat).ToList
    End Function

    ''' <summary>
    ''' Retorna la llista de factures d'un proveidor concret
    ''' </summary>
    ''' <param name="idProveidor">Identificador del client</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetfacturesProList(idProveidor As Integer) As List(Of FacturaPro)
        Return (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.Proveidor IsNot Nothing AndAlso x.Proveidor.ProveidorId = idProveidor).ToList
    End Function

    ''' <summary>
    ''' Obté una Factura de proveidor a partir del seu Id
    ''' </summary>
    ''' <param name="facturaProID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetFacturaProById(facturaProID As Long) As FacturaPro
        Return (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.FacturaProId = facturaProID).FirstOrDefault
    End Function

    ''' <summary>
    ''' Crea o modifica una factura a la base de dades
    ''' </summary>
    ''' <param name="factura">Factura a actualitzar (si idFactura=-1, s'afegeix)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(factura As FacturaPro) As Boolean
        Dim f As FacturaPro

        If factura.FacturaProId < 0 Then
            f = factura
            f.FacturaProId = General.GetNewIdFacturaPro
            General.ctx.FACTURES_PRO.Add(f)
        Else
            f = (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.FacturaProId = factura.FacturaProId).FirstOrDefault
            If f Is Nothing Then
                f = factura
                f.FacturaProId = General.GetNewIdFactura
                f.Proveidor = (From PR As Proveidor In ctx.PROVEIDORS Select PR).FirstOrDefault
                f.ProveidorId = f.Proveidor.ProveidorId
                General.ctx.FACTURES_PRO.Add(f)
            Else

                f.FromObject(factura)

                ctx.Entry(f).State = EntityState.Modified
            End If
        End If

        General.ctx.SaveChanges()

        Return True
    End Function

    Public Shared Function Actualitza(Lin As FacturaProLin) As Boolean
        If Lin Is Nothing Then Return False
        Dim OldLin As FacturaProLin
        Dim p As FacturaPro = GetFacturaProById(Lin.FacturaProId)


        'Si Lin = -1, crear nova, si no, actualitzar.

        OldLin = (From L As FacturaProLin In p.LiniesFactura Where L.Linia = Lin.Linia).FirstOrDefault

        If Lin.Linia = -1 OrElse OldLin Is Nothing Then
            Try
                Lin.Linia = CByte((From L As FacturaProLin In p.LiniesFactura
                                        Select L.Linia).Max + 1)
            Catch ex As Exception
                Lin.Linia = 1
            End Try

            p.LiniesFactura.Add(Lin)

            ctx.Entry(p).State = EntityState.Modified

        Else
            OldLin.ArticleId = Lin.ArticleId
            OldLin.Quantitat = Lin.Quantitat
            OldLin.Preu = Lin.Preu
            OldLin.PercIVA = Lin.PercIVA
            ctx.Entry(p).State = EntityState.Modified
        End If


        General.ctx.SaveChanges()
        Return True

    End Function

    ''' <summary>
    ''' Elimina una factura de la base de dades
    ''' </summary>
    ''' <param name="facturaId">Id de la factura a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarFacturaPro(facturaId As Long) As Boolean
        Dim f As FacturaPro
        f = (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.FacturaProId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            General.ctx.FACTURES_PRO.Remove(f)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function

    ''' <summary>
    ''' Elimina una factura de la base de dades
    ''' </summary>
    ''' <param name="FacturaProId">Id de la factura a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarLiniaFacturaPro(FacturaProId As Long, Linia As Byte) As Boolean

        Dim p As FacturaPro = GetFacturaProById(FacturaProId)
        If p Is Nothing Then Return False
        Dim l As FacturaProLin = (From xlin As FacturaProLin In p.LiniesFactura
                                  Where xlin.Linia = Linia).FirstOrDefault
        If l Is Nothing Then Return False
        Dim ok As Boolean = p.LiniesFactura.Remove(l)
        ctx.Entry(p).State = EntityState.Modified
        ctx.SaveChanges()
        Return ok
    End Function

    ''' <summary>
    ''' Passa la factura a estat 'Facturada' i genera els pagaments en base a la informació de la factura 
    ''' </summary>
    ''' <param name="facturaId"></param>
    ''' <returns>Retorna la factura actualitzada </returns>
    ''' <remarks></remarks>
    Public Shared Function FacturarFacturaPro(facturaId As Long) As FacturaPro
        Dim f As FacturaPro
        f = (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.FacturaProId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            If f.EstatFactura = EnumEstatsFactura.Generada Then
                'Generar los cobros

                f.Pagaments = New List(Of Pagament)
                Dim suma As Double = 0
                Dim imp As Double

                If f.NumVenciments = 0 Then
                    f.NumVenciments = 1

                End If

                For i As Integer = 1 To f.NumVenciments
                    If i < f.NumVenciments Then
                        imp = ((f.Import * 100) \ f.NumVenciments) * 100
                        suma += imp
                    Else
                        ' Si es l'ultim venciment, s'haurà de pagar la resta que quedi pendent
                        imp = f.Import - suma

                    End If

                    Dim dv As Date = f.Data.AddMonths(i - 1)
                    dv = DateSerial(dv.Year, dv.Month, dv.Day)

                    f.Pagaments.Add(New Pagament With {
                                         .Proveidor = f.Proveidor,
                                         .ProveidorId = f.ProveidorId,
                                         .DataFactura = f.Data,
                                         .DataVenciment = dv,
                                         .UltimaReclamacio = Date.MinValue,
                                         .Estat = EnumEstatsTransaccio.NoCobrat,
                                         .FacturaPro = f,
                                         .FacturaProId = f.FacturaProId,
                                         .FormaPagament = f.FormaPagament,
                                         .Numero = i,
                                         .Import = imp})
                Next

                f.EstatFactura = EnumEstatsFacturaPro.Facturada
                ctx.Entry(f).State = EntityState.Modified
                General.ctx.SaveChanges()
            End If
        End If
        Return f
    End Function

    ''' <summary>
    ''' Si la l'estat de la factura és 'Facturada', passa a l'estat 'Generada' i s'eliminen els pagaments.
    ''' En cas contrari, no té efecte
    ''' </summary>
    ''' <param name="facturaId"></param>
    ''' <returns>Retorna la factura actualitzada </returns>
    ''' <remarks>Si la factura té pagaments en estat 'Processat', no es canviarà l'estat. Primer s'han de canviar els estats dels pagaments.</remarks>
    Public Shared Function AnularFacturaPro(facturaId As Long) As FacturaPro
        Dim f As FacturaPro
        f = (From x As FacturaPro In General.ctx.FACTURES_PRO Where x.FacturaProId = facturaId).FirstOrDefault
        If f IsNot Nothing Then
            If f.EstatFactura = EnumEstatsFactura.Facturada Then
                'Si està en estat facturada, no té pagaments realitzats encara. Llavors, es poden eliminar els cobraments i
                'passar a l'estat Acceptada

                If f.Pagaments IsNot Nothing Then
                    f.Pagaments.Clear()
                End If

                f.EstatFactura = EnumEstatsFactura.Acceptada

                General.ctx.SaveChanges()
            End If
        End If

        Return f
    End Function

    Public Shared Function ObtenirDadesFile(ByVal id As Long, ByRef dades As Byte(), ByRef tipus As String) As Boolean

        Dim F As Document = ctx.DOCUMENTS.Where(Function(d) d.Id = id).FirstOrDefault
        If F Is Nothing Then Return False

        Dim key As Byte() = CompressionUtilites.StringAsciiToBytes(" `+h	?9?????8[{???ua?,?n??")
        Dim iv As Byte() = CompressionUtilites.MD5HashToByte("6260bb059ec9fcdda3ec271268e953ba")

        Dim str As String = F.dades
        dades = System.Text.Encoding.Default.GetBytes(str)
        dades = CompressionUtilites.ZipDecryptAndDecompress(dades, key, iv)
        tipus = F.ContentType

        Return True

    End Function

    Public Shared Function AcceptarFile(ByVal id As Long) As FacturaPro

        Dim F As Document = ctx.DOCUMENTS.Where(Function(d) d.Id = id).FirstOrDefault
        If F Is Nothing Then Return Nothing

        F.Estat = EnumEstatsDocument.Acceptat
        ctx.Entry(F).State = EntityState.Modified

        'Generem la factura de proveedor amb les dades que tenim
        Dim fac As New FacturaPro
        With fac
            .ProveidorId = F.ProveidorId
            .Data = F.DataFactura
            .EsDespesa = False
            .Usuari = 0

            .FacturaProId = GetNewIdFacturaPro()
            .Descripcio = Recursos.camps_factura & " " & .FacturaProId
        End With

        fac.LiniesFactura.Add(New FacturaProLin With {
                              .Descripcio = fac.Descripcio,
                              .FacturaPro = fac,
                              .FacturaProId = fac.FacturaProId,
                              .Linia = 1,
                              .PercDescompte = 0,
                              .Quantitat = 1,
                              .PercIVA = 0.21,
                              .Preu = F.Import / .PercIVA})

        ctx.FACTURES_PRO.Add(fac)
        ctx.SaveChanges()
        Return fac

    End Function

    Public Shared Function DenegarFile(id As Long, comentario As String, enviarmail As Boolean) As Boolean
        Dim F As Document = ctx.DOCUMENTS.Where(Function(d) d.Id = id).FirstOrDefault
        If F Is Nothing Then Return False

        F.Observacions = comentario

        If enviarmail Then
            F.Estat = EnumEstatsDocument.RebutjatEnviarEmail
        Else
            F.Estat = EnumEstatsDocument.Rebutjat
        End If

        ctx.SaveChanges()

        Return True
    End Function

#End Region

#Region "Gestió Pagaments"

    ''' <summary>
    ''' Retorna la llista de pagaments entre les dates seleccionades.
    ''' </summary>
    ''' <param name="DataInicial">Data inicial</param>
    ''' <param name="DataFinal">Data Final</param>
    ''' <param name="estat">(Opcional) Estat del pagament. Si no s'especifica, es retornen tots</param>
    ''' <param name="proveidorId">(Opcional) Filtre per proveïdor. Si no s'especifica, es retornen tots</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetpagamentList(DataInicial As Date, DataFinal As Date, Optional estat As EnumEstatsTransaccio = EnumEstatsTransaccio.NoCobrat, Optional proveidorId As Integer = -1) As List(Of Pagament)

    End Function

    ''' <summary>
    ''' Obté un pagament a partir del seu Id
    ''' </summary>
    ''' <param name="pagamentID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function GetPagamentById(pagamentID As Long) As Pagament

    End Function

    ''' <summary>
    ''' Crea o modifica un pagament a la base de dades
    ''' </summary>
    ''' <param name="pagament">Pagament a actualitzar (si id=-1, s'afegeix)</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Shared Function Actualitza(pagament As Pagament) As Boolean

    End Function

    ''' <summary>
    ''' Elimina un pagament de la base de dades
    ''' </summary>
    ''' <param name="pagamentId">Id del pagament a esborrar</param>
    ''' <returns>'True' si s'ha pogut esborrar i 'False' si no </returns>
    ''' <remarks></remarks>
    Public Shared Function EliminarPagament(pagamentId As Integer) As Boolean

    End Function

    ''' <summary>
    ''' Passa el pagament a estat 'Processat' i canvia l'estat de la factura en funció del nombre de pagaments pendents. 
    ''' </summary>
    ''' <param name="pagamentId"></param>
    ''' <returns>Retorna el pagament actualitzat </returns>
    ''' <remarks></remarks>
    Public Shared Function ProcessarPagament(pagamentId As Integer) As Pagament

    End Function

    ''' <summary>
    ''' Si la l'estat del pagament és 'Processat', passa a l'estat 'No Processat' en cas contrari, no té efecte.
    ''' </summary>
    ''' <param name="pagamentId"></param>
    ''' <returns>Retorna el pagament actualitzat </returns>
    ''' <remarks></remarks>
    Public Shared Function CancelarPagament(pagamentId As Integer) As Pagament

    End Function

#End Region

#Region "Gestió Artícles"

    Public Shared Function GetArticlesList(ByVal orden As String, ByVal filtronombre As String) As List(Of Article)
        Try
            Dim q = (From a As Article In ctx.ARTICLES Select a)

            If orden Is Nothing Then orden = ""

            If Not String.IsNullOrEmpty(filtronombre) Then
                q = q.Where(Function(x) x.Nom.ToLower.Contains(filtronombre.ToLower))
            End If

            Select Case orden.ToLower
                Case "id_desc"
                    q = q.OrderByDescending(Function(c) c.ArticleId)
                Case "nom"
                    q = q.OrderBy(Function(c) c.Nom)
                Case "nom_desc"
                    q = q.OrderByDescending(Function(c) c.Nom)
                Case Else
                    q = q.OrderBy(Function(c) c.ArticleId)
            End Select

            Return q.ToList

        Catch ex As Exception
            Return New List(Of ModelLayer.Article)
        End Try
    End Function

    Public Shared Function GetArticleById(articleID As Long) As Article
        Return General.ctx.ARTICLES.Find(articleID)
    End Function

    Public Shared Function Actualitza(article As Article) As Boolean
        Dim a As Article

        If article.ArticleId <= 0 Then
            a = article
            General.ctx.ARTICLES.Add(a)
        Else

            a = General.ctx.ARTICLES.Find(article.ArticleId)
            If a Is Nothing Then
                a = article
                General.ctx.ARTICLES.Add(a)
            Else
                a.FromObject(article)
            End If

            ctx.Entry(a).State = EntityState.Modified
        End If

        General.ctx.SaveChanges()

        Return True
    End Function

    Public Shared Function EliminarArticle(articleId As Long) As Boolean
        Dim a As Article
        a = (From x As Article In General.ctx.ARTICLES Where x.ArticleId = articleId).FirstOrDefault
        If a IsNot Nothing Then
            General.ctx.ARTICLES.Remove(a)
            General.ctx.SaveChanges()
            Return True
        Else
            Return False
        End If
    End Function
#End Region

#Region "Estadístiques"

    Public Shared Function GetDadessBalanc(dataIni As Date, dataFi As Date) As InformeBalanc
        Dim d As New InformeBalanc

        d.HasData = True
        dataIni = New Date(dataIni.Year, dataIni.Month, dataIni.Day, 0, 0, 0)
        dataFi = New Date(dataFi.Year, dataFi.Month, dataFi.Day, 23, 59, 59)


        'Clients
        Dim L1 As List(Of FacturaCli)
        Dim L2 As List(Of FacturaPro)

        Try
            L1 = (From f As FacturaCli In ctx.FACTURES_CLI Where f.Data >= dataIni And
                                            f.Data <= dataFi And
                                            (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                   f.EstatFactura = EnumEstatsFactura.CobradaParcialment Or
                                                   f.EstatFactura = EnumEstatsFactura.Cobrada)
                                            Select f).ToList


            d.TotalFacturat = Math.Round((From f As FacturaCli In L1 Select f.Import).Sum, 2)
        Catch ex As Exception
            d.TotalFacturat = 0
        End Try

        Try
            d.TotalCobraments = Math.Round((From c As Cobrament In ctx.COBRAMENTS Where c.DataVenciment >= dataIni And
                             c.DataVenciment <= dataFi
                             Select c.Import).Sum, 2)
        Catch ex As Exception
            d.TotalCobraments = 0
        End Try

        Try
            L1 = (From f As FacturaCli In ctx.FACTURES_CLI Where f.Data >= dataIni And
                             f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                   f.EstatFactura = EnumEstatsFactura.CobradaParcialment)
                                               Select f).ToList


            d.TotalPteCobrament = Math.Round((From f As FacturaCli In L1
                             Select f.Import - (From c As Cobrament In ctx.COBRAMENTS
                                                Where c.FacturaCliId = f.FacturaCliId
                                                Select c.Import).Sum).Sum, 2)
        Catch ex As Exception
            d.TotalPteCobrament = 0
        End Try

        'Proveidors

        Try
            L2 = (From f As FacturaPro In ctx.FACTURES_PRO Where f.Data >= dataIni And
                             f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                   f.EstatFactura = EnumEstatsFactura.CobradaParcialment Or
                                                   f.EstatFactura = EnumEstatsFactura.Cobrada)
                             Select f).ToList

            d.TotalFacturesPro = Math.Round((From f As FacturaPro In L2 Select f.Import).Sum, 2)
        Catch ex As Exception
            d.TotalFacturesPro = 0
        End Try

        Try
            d.TotalPagos = Math.Round((From c As Pagament In ctx.PAGAMENTS Where c.DataVenciment >= dataIni And
                             c.DataVenciment <= dataFi
                             Select c.Import).Sum, 2)
        Catch ex As Exception
            d.TotalPagos = 0
        End Try

        Try
            L2 = (From f As FacturaPro In ctx.FACTURES_PRO Where f.Data >= dataIni And
                             f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                   f.EstatFactura = EnumEstatsFactura.CobradaParcialment Or
                                                   f.EstatFactura = EnumEstatsFactura.Cobrada)
                             Select f).ToList


            d.TotalPtePago = Math.Round((From f As FacturaPro In L2
                             Select f.Import - (From c As Pagament In ctx.PAGAMENTS
                                                Where c.FacturaProId = f.FacturaProId
                                                Select c.Import).Sum).Sum, 2)
        Catch ex As Exception
            d.TotalPtePago = 0
        End Try

        Return d

    End Function

    Public Shared Function GetDadessLlibreIva(dataIni As Date, dataFi As Date) As List(Of InformeLlibreIVA)

        Dim L As New List(Of InformeLlibreIVA)
        dataIni = New Date(dataIni.Year, dataIni.Month, dataIni.Day, 0, 0, 0)
        dataFi = New Date(dataFi.Year, dataFi.Month, dataFi.Day, 23, 59, 59)

        Try
            'Afegim el IVA de les factures de clients, Repercutit

            L.AddRange(
                (From f As FacturaCli In ctx.FACTURES_CLI
                     Where f.Data >= dataIni And
                                 f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                       f.EstatFactura = EnumEstatsFactura.CobradaParcialment)
                    Select New InformeLlibreIVA With {
                        .Data = f.Data,
                        .Concepte = String.Format("({0}) - {1}", f.FacturaCliId, f.Descripcio),
                        .Suportat = 0,
                        .Repercutit = f.LiniesFactura.Sum(Function(ll) ll.IVALinia)
                    }).ToList
            )

        Catch ex As Exception

        End Try


        Try
            'Afegim l'iva de les factures de proveidors, Suportat

            L.AddRange(
                (From f As FacturaPro In ctx.FACTURES_PRO
                     Where f.Data >= dataIni And
                                 f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                                       f.EstatFactura = EnumEstatsFactura.CobradaParcialment)
                    Select New InformeLlibreIVA With {
                        .Data = f.Data,
                        .Concepte = String.Format("({0}) - {1}", f.FacturaProId, f.Descripcio),
                        .Suportat = f.LiniesFactura.Sum(Function(ll) ll.IVALinia),
                        .Repercutit = 0
                    }).ToList
            )


        Catch ex As Exception

        End Try

        Return L

    End Function

    Public Shared Function GetDadesFacturacio(dataIni As Date, dataFi As Date) As List(Of InformeEstadistFact)

        Dim L As New List(Of InformeEstadistFact)

        dataIni = New Date(dataIni.Year, dataIni.Month, dataIni.Day, 0, 0, 0)
        dataFi = New Date(dataFi.Year, dataFi.Month, dataFi.Day, 23, 59, 59)

        Try
            Dim ListFac As List(Of FacturaCli) = (From f As FacturaCli In ctx.FACTURES_CLI
                        Where f.Data >= dataIni And
                        f.Data <= dataFi And (f.EstatFactura = EnumEstatsFactura.Facturada Or
                                            f.EstatFactura = EnumEstatsFactura.CobradaParcialment)
                        Select f).ToList


            For Each Fac As FacturaCli In ListFac
                Dim xLin As InformeEstadistFact = (From x As InformeEstadistFact In L Where x.Any = Fac.Data.Year).FirstOrDefault
                If xLin Is Nothing Then
                    xLin = New InformeEstadistFact
                    xLin.Any = Fac.Data.Year
                    L.Add(xLin)
                End If

                Select Case Fac.Data.Month
                    Case 1 : xLin.Mes1 += Math.Round(Fac.Import, 2)
                    Case 2 : xLin.Mes2 += Math.Round(Fac.Import, 2)
                    Case 3 : xLin.Mes3 += Math.Round(Fac.Import, 2)
                    Case 4 : xLin.Mes4 += Math.Round(Fac.Import, 2)
                    Case 5 : xLin.Mes5 += Math.Round(Fac.Import, 2)
                    Case 6 : xLin.Mes6 += Math.Round(Fac.Import, 2)
                    Case 7 : xLin.Mes7 += Math.Round(Fac.Import, 2)
                    Case 8 : xLin.Mes8 += Math.Round(Fac.Import, 2)
                    Case 9 : xLin.Mes9 += Math.Round(Fac.Import, 2)
                    Case 10 : xLin.Mes10 += Math.Round(Fac.Import, 2)
                    Case 11 : xLin.Mes11 += Math.Round(Fac.Import, 2)
                    Case 12 : xLin.Mes12 += Math.Round(Fac.Import, 2)
                End Select
            Next







            'NO FUNCIONAAAAAA ¿¿?? :(
            'L.AddRange(
            '    (From f As FacturaCli In ListFac
            '        Group f By f.Data.Year Into Lista = Group
            '        Select New InformeEstadistFact With {
            '            .Any = Year,
            '            .Mes1 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 1).Sum(Function(x) x.Import),
            '            .Mes2 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 2).Sum(Function(x) x.Import),
            '            .Mes3 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 3).Sum(Function(x) x.Import),
            '            .Mes4 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 4).Sum(Function(x) x.Import),
            '            .Mes5 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 5).Sum(Function(x) x.Import),
            '            .Mes6 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 6).Sum(Function(x) x.Import),
            '            .Mes7 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 7).Sum(Function(x) x.Import),
            '            .Mes8 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 8).Sum(Function(x) x.Import),
            '            .Mes9 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 9).Sum(Function(x) x.Import),
            '            .Mes10 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 10).Sum(Function(x) x.Import),
            '            .Mes11 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 11).Sum(Function(x) x.Import),
            '            .Mes12 = Lista.Where(Function(x) x IsNot Nothing And x.Data.Month = 12).Sum(Function(x) x.Import)}).ToList
            '    )


        Catch ex As Exception

        End Try



        Return L
    End Function


#End Region


#Region "No Persistents"
    Public Shared Function GetTipusUsuari() As List(Of KeyValuePair(Of Integer, String))
        Dim L As New List(Of KeyValuePair(Of Integer, String))
        L.Add(New KeyValuePair(Of Integer, String)(0, "Common"))
        L.Add(New KeyValuePair(Of Integer, String)(1, "Admin"))
        Return L
    End Function
#End Region




End Class
